% Description:
%    This is a toy example of PCA, more details of the code are on 
%    https://kiwi.ecn.purdue.edu/rhea/index.php/PCA
% 
% Author:
%    Tian Zhou (zhou338@purdue.edu)
%    Intelligent System and Assistive Technology (ISAT) Lab
%    Industrial Engineering, Purdue University
%     
% Date: 
%     2014/3/30
% 
% Reference:
%     Smith, L. I. (2002). A tutorial on principal components analysis. Cornell University, USA, 51, 52.

close all
clear
clc

% generate original data set
x = 1:0.5:10;
y = x + 2*rand(1,19);
% save y;
% load y;
figure
plot(x,y,'k*')
grid on
xlabel('dimension 1')
ylabel('dimension 2')
title('original data set')

% subtract the original data with its average
x = x - mean(x);
y = y - mean(y);
figure
plot(x,y,'k*')
grid on
xlabel('dimension 1')
ylabel('dimension 2')
title('shifted data set')
axis([-6 6 -6 6])
axis square

% get the covariance matrix and the eigenvalue and eigenvectors
cov = cov(x,y);
[V,D] = eig(cov);

% show the direction of the two eigen vectors
e1 = -5:0.1:5;
ev1 = V(2,1)/V(1,1)*e1;
ev2 = V(2,2)/V(1,2)*e1;
beta = (x*x')^-1*x*y';
regress_y = beta.*e1;
hold on;
plot(e1,ev1,'r')
plot(e1,ev2,'b')
plot(e1,regress_y,'m')
legend('2-d data sets','eigenvector for eigenvalue 0.2315', ...
    'eigenvector for eigenvalue 16.84','least-square fit line');
axis([-6 6 -6 6])
axis square
grid on


% project the data onto the axis decided by the eigenvecotr corresponding
% to the larger eigenvalue
figure;
plot(x,y,'k*',e1,ev2,'k');
proj_axis = V(:,2);
proj_matrix = proj_axis*(proj_axis'*proj_axis)^-1*proj_axis';
orig_data = [x;y];
projected_data = proj_matrix*orig_data;
projection_error1 = projected_data - orig_data;
hold on
plot(projected_data(1,:),projected_data(2,:),'b*');
axis([-6 6 -6 6])
axis square

% plot the error
for i = 1:length(x)
    hold on
    plot ([x(i) projected_data(1,i)], [y(i) projected_data(2,i)], 'r'); 
end %for

title('Projection onto the eigenvector which corresponds to the large eigenvalue')
legend('Original data','Principle direction','Projected data','Error of projection');

% calculate the error of projection
sum_err_1 = 0;
for i = 1:length(projection_error1)
    sum_err_1 = sum_err_1 + sqrt(projection_error1(1,i)^2+projection_error1(2,i)^2);
end %for


% show the PCA reduced data in 1D
PCA_data = proj_axis' * orig_data;

figure;
hold off;
plot(PCA_data,zeros(length(PCA_data)), 'b*');
title('PCA reduced data')
xlabel('The reduced axis');


% project the data onto the axis decided by the eigenvecotr corresponding
% to the smaller eigenvalue
figure;
plot(x,y,'k*',e1,ev1,'k');
proj_axis = V(:,1);
proj_matrix = proj_axis*(proj_axis'*proj_axis)^-1*proj_axis';
orig_data = [x;y];
projected_data = proj_matrix*orig_data;
projection_error2 = projected_data - orig_data;
hold on
plot(projected_data(1,:),projected_data(2,:),'b*');
axis([-6 6 -6 6])
axis square

% plot the error
for i = 1:length(x)
    hold on
    plot ([x(i) projected_data(1,i)], [y(i) projected_data(2,i)], 'r'); 
end %for

% title('Projection onto the eigenvector which corresponds to the small eigenvalue')
legend('Original data','Principle direction','Projected data','Error of projection');

% calculate the error of projection
sum_err_2 = 0;
for i = 1:length(projection_error2)
    sum_err_2 = sum_err_2 + sqrt(projection_error2(1,i)^2+projection_error2(2,i)^2);
end %for

PCA_data = proj_axis' * orig_data;

% show the error of the two projections
sum_err_1
sum_err_2


